/*	main.c

	Copyright 2008 Fred Cooke

	This file is part of the FreeEMS project.

	FreeEMS software is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	FreeEMS software is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with any FreeEMS software.  If not, see <http://www.gnu.org/licenses/>.

	We ask that if you make any changes to this file you send them upstream to us at admin@diyefi.org

	Thank you for choosing FreeEMS to run your engine! */

#include "main.h"

/* Correctly placed in memory due to compiler/linker directives in memory.x and the linker script.	*/
/* This is the FULL table of length 0xFF starting at 0xFF00 and ending at 0xFFFF, redirected with	*/
/* jumps to the offset location by the serial monitor starting at 0xF700 and ending at 0xF800		*/
/* http://m68hc11.serveftp.org/wiki/index.php/FAQ:Interrupts										*/
const interruptTable _vectors[] VECTORS = {
/* 0xFF00 to 0xFF0F */
/*	The first row are NOT actually interrupts at all, just a wasted 16 bytes for tidiness */
/*	UISR,			UISR,			UISR,			UISR,			UISR,			UISR,			UISR,			UISR,		*/
/*	Reserved		Reserved		Reserved		Reserved		Reserved		Reserved		Reserved		Reserved	*/
/*																																*/

/* 0xFF10 to 0xFF1F */
	UISR,			UISR,			UISR,			UISR,			UISR,			UISR,			UISR,			UISR,
/*	Spurious		Reserved		Reserved		Reserved		Reserved		Reserved		Reserved		Reserved	*/
/*																																*/

/* 0xFF20 to 0xFF2F */
	UISR,			UISR,			UISR,			UISR,			UISR,			UISR,			UISR,			UISR,
/*	Reserved		Reserved		Reserved		Reserved		Reserved		Reserved		Reserved		Reserved	*/
/*																																*/

/* 0xFF30 to 0xFF3F */
	UISR,			UISR,			UISR,			UISR,			UISR,			UISR,			UISR,			UISR,
/*	Reserved		Reserved		Reserved		Reserved		Reserved		Reserved		Reserved		Reserved	*/
/*																																*/

/* 0xFF40 to 0xFF4F */
	UISR,			UISR,			UISR,			UISR,			UISR,			UISR,			UISR,			UISR,
/*	Reserved		Reserved		Reserved		Reserved		Reserved		Reserved		Reserved		Reserved	*/
/*																																*/

/* 0xFF50 to 0xFF5F */
	UISR,			UISR,			UISR,			UISR,			UISR,			UISR,			UISR,			UISR,
/*	Reserved		Reserved		Reserved		Reserved		Reserved		Reserved		Reserved		Reserved	*/
/*																																*/

/* 0xFF60 to 0xFF6F */
	UISR,			UISR,			UISR,			UISR,			UISR,			UISR,			UISR,			UISR,
/*	RAM violation	XGATEsoft error	XGATE 7			XGATE 6			XGATE 5			XGATE 4			XGATE 3			XGATE 2		*/
/*																																*/

/* 0xFF70 to 0xFF7F */
	UISR,			UISR,			PIT3ISR,		PIT2ISR,		IgnitionFireISR,IgnitionDwellISR,UISR,			UISR,
/*	XGATE 1			XGATE 0			PIT 3			PIT 2			PIT 1			PIT 0			Reserved		API			*/
/*																	Fire Coil		Dwell Coil									*/

/* 0xFF80 to 0xFF8F */
	UISR,			UISR,			UISR,			UISR,			UISR,			UISR,			UISR,			PortPISR,
/*	Low Voltage		IIC1			SCI5			SCI4			SCI3			SCI2			PWM ESDown		Port P		*/
/*																																*/

/* 0xFF90 to 0xFF9F */
	UISR,			UISR,			UISR,			UISR,			UISR,			UISR,			UISR,			UISR,
/*	CAN4 Tx			CAN4 Rx			CAN4 Errors		CAN4 Wakeup		CAN3 Tx			CAN3 Rx			CAN3 Errors		CAN3 Wakeup	*/
/*																																*/

/* 0xFFA0 to 0xFFAF */
	UISR,			UISR,			UISR,			UISR,			UISR,			UISR,			UISR,			UISR,
/*	CAN2 Tx			CAN2 Rx			CAN2 Errors		CAN2 Wakeup		CAN1 Tx			CAN1 Rx			CAN1 Errors		CAN1 Wakeup	*/
/*																																*/

/* 0xFFB0 to 0xFFBF */
	UISR,			UISR,			UISR,			UISR,			UISR,			UISR,			UISR,			UISR,
/*	CAN0 Tx			CAN0 Rx			CAN0 Errors		CAN0 Wakeup		FLASH			EEPROM			SPI2			SPI1		*/
/*																																*/

/* 0xFFC0 to 0xFFCF */
	UISR,			UISR,			UISR,			UISR,			UISR,			UISR,			PortHISR,		PortJISR,
/*	IIC0			Reserved		CRG self clock	CRG PLL lock	PAB Overflow	ModDwnCtrUF		Port H			Port J		*/
/*																																*/

/* 0xFFD0 to 0xFFDF */
	UISR,			UISR,			UISR,			UISR,			UISR,			UISR,			UISR,			UISR,
/*	ATD1			ATD0?			SCI1			SCI0			SPI0			PAIE			PAA OF			ECT OF		*/
/*																																*/

/* 0xFFE0 to 0xFFEF */
	Injector6ISR,	Injector5ISR,	Injector4ISR,	Injector3ISR,	Injector2ISR,	Injector1ISR,	SecondaryRPMISR,PrimaryRPMISR,	
/*	ECT7			ECT6			ECT5			ECT4			ECT3			ECT2			ECT1			ECT0		*/
/*	Injector 6		Injector 5		Injector 4		Injector 3		Injector 2		Injector 1		Secondary RPM	Primary RPM	*/

/* 0xFFF0 to 0xFFFF */
	RTIISR,			IRQISR,			XIRQISR,		UISR,			UISR,			UISR,			UISR,			_start
/*	RTI				IRQ				XIRQ			SWI				UnimpInstruct	COP Reset		ClockReset		SystemReset	*/
/*																													Entry point */
};

void inject(int channel)
{
	// Toggle the correct cylinders channel on or off.
	if(channel == 0){
		Injector1();
	}else if(channel == 1){
		Injector2();
	}else if(channel == 2){
		Injector3();
	}else if(channel == 3){
		Injector4();
	}else if(channel == 4){
		Injector5();
	}else if(channel == 5){
		Injector6();
	}
}

void Injector1(void)
{
	if(localFuelCut == 0){
		PTT ^= 0x04;
		if(localStagedInjection > 0){
			PORTK ^= 0x01;
		}
	}
	if(localSparkCut == 0){
		if(phase == 0){
			PORTB ^= 0x01;
		}else{
			PORTB ^= 0x40;
		}
	}
}

void Injector2(void)
{
	if(localFuelCut == 0){
		PTT ^= 0x08;
		if(localStagedInjection > 0){
			PORTK ^= 0x02;
		}
	}
	if(localSparkCut == 0){
		if(phase == 0){
			PORTB ^= 0x02;
		}else{
			PORTB ^= 0x80;
		}
	}
}

void Injector3(void)
{
	if(localFuelCut == 0){
		PTT ^= 0x10;
		if(localStagedInjection > 0){
			PORTK ^= 0x04;
		}
	}
	if(localSparkCut == 0){
		if(phase == 0){
			PORTB ^= 0x04;
		}else{
			PORTA ^= 0x01;
		}
	}
}

void Injector4(void)
{
	if(localFuelCut == 0){
		PTT ^= 0x20;
		if(localStagedInjection > 0){
			PORTK ^= 0x08;
		}
	}
	if (localSparkCut == 0) {
		if(phase == 0){
			PORTB ^= 0x08;
		}else{
			PORTA ^= 0x02;
		}
	}
}

void Injector5(void)
{
	if(localFuelCut == 0){
		PTT ^= 0x40;
		if(localStagedInjection > 0){
			PORTK ^= 0x10;
		}
	}
	if(localSparkCut == 0){
		if(phase == 0){
			PORTB ^= 0x10;
		}else{
			PORTA ^= 0x04;
		}
	}
}

void Injector6(void)
{
	if(localFuelCut == 0){
		PTT ^= 0x80;
		if(localStagedInjection > 0){
			PORTK ^= 0x20;
		}
	}
	if(localSparkCut == 0){
		if(phase == 0){
			PORTB ^= 0x20;
		}else{
			PORTA ^= 0x08;
		}
	}
}

int  main(){
	// Set everything up.
	init();
	
	// intitial PWM settings
	PWMPER0 = 3;
	PWMPER1 = 7;
	PWMPER2 = 15;
	PWMPER3 = 31;
	PWMPER4 = 63;
	PWMPER5 = 127;
	
	PWMDTY5 = 1;
	
	// Run forever repeating.
	while(TRUE){
		/* Flash the rest of port A (3 are initialised off and one on, so they alternate) */
		PORTA ^= 0xF0;

		// Take a copy of the vars before each loop such that the LEDs stay in consistent state during changes.
		unsigned char localEvents = combustionEventsPerEngineCycle;
		unsigned char localPorts = ports;

		// Flash the injector channel leds on and off repeatedly in a strobed sequence.
		int i;
		for(i=0;i < localEvents;i++)
		{
			int channel;
			
			// update the toggle variables before each inner loop iteration
			localStagedInjection = stagedInjection;
			localSparkCut = sparkCut;
			localFuelCut = fuelCut;

			// on once per cycle for J0 (tacho)
			PORTJ ^= 0x01;
			
			// set the phase and channel based on cylinder count and port count
			if(localEvents <= 6 || (i + localPorts) < localEvents){
				phase = 0;
				channel = i;
			}else{
				phase = 1;
				channel = i - localPorts;
			}

			// Turn channel on.
			inject(channel);

			// Snooze for dual adc dependent time.
			sleep2();
			
			// and off again too (tacho)
			PORTJ ^= 0x01;
			
			// PWM experimentation
			PWMPER0 += 1;
			PWMPER1 += 1;
			PWMPER2 += 1;
			PWMPER3 += 1;
			PWMPER4 += 1;
			PWMPER5 += 1;
			
			PWMDTY0 = PWMPER0 / 2;
			PWMDTY1 = PWMPER1 / 4;
			PWMDTY2 = PWMPER2 / 8;
			PWMDTY3 = (3 * PWMPER3) / 4;
			PWMDTY4 = (7 * PWMPER4) / 8;
			PWMDTY5 -= 1;
			
			// Snooze that time again.
			sleep2();
			
			// Turn channel off.
			inject(channel);
			
			// Next cylinder...			
		}
	}
}
